%-- This code has been realized to illustrate neural network concepts during 
%-- the workshop organized by the "club EEA" which happened in March 2018 
%-- https://rnn2018lyon.sciencesconf.org/
%--
%-- This code has been strongly inspired by the Mooc of Andrew Ng on coursera
%-- https://www.coursera.org/learn/machine-learning/
%-- 
%-- The application treated here concerns the automatic recognition of 
%-- digits drawn by hand. This code is not protected by any license, you 
%-- can re-use it freely without any restriction. Please do not forget to
%-- cite this event for any use of the code.

function strTxt = computeTxtInfo()

    %-- mnist database location
    local_data_path = '../data/';
    local_param_path = '../param/';


   %-- Load parameters and corresponding database
    if (exist([local_param_path,'param_mnist_new.mat'],'file'))
        load([local_param_path,'param_mnist_new.mat']);
        load([local_data_path,'mnist_new.mat']);
    else
        load([local_param_path,'param_mnist.mat']);
        load([local_data_path,'mnist.mat']);
    end
    widthDigit = size(training.images,2);
    heightDigit = size(training.images,1);

    %-- Create X matrix
    X = zeros(size(testing.images,3),widthDigit*heightDigit);
    for k=1:size(testing.images,3)
        digit = testing.images(:,:,k);
        X(k,:) = digit(:)';
    end
    y = testing.labels;
    y(y==0) = 10;   %-- the digit 0 is identified as label 10


    %-- Evaluate the overall performance of the learned method from the full testing database
    pred = nn.predict(Theta1, Theta2, X);
    accuracy = mean(double(pred == y)) * 100;
    
    
    %-- Evaluate the performance of the learned method from the full
    %-- testing database for each digit
    accuracyDigit = zeros(1,10);
    y(y==10) = 0;
    pred(pred==10) = 0;
    for k=0:9
        ind = (y==k);
        yD = y(ind);
        predD = pred(ind);
        accuracyDigit(k+1) = mean(double(predD == yD)) * 100;
    end

    
    %-- Compute text structure information
    strTxt = { sprintf('Overall accuracy: %.1f %%\n',accuracy),...
        sprintf('Samples training: %d ',training.count),...
        sprintf('Samples test: %d \n',testing.count),...        
        sprintf('    digit 0: %.1f %%',accuracyDigit(1)),...
        sprintf('    digit 1: %.1f %%',accuracyDigit(2)),...
        sprintf('    digit 2: %.1f %%',accuracyDigit(3)),...
        sprintf('    digit 3: %.1f %%',accuracyDigit(4)),...
        sprintf('    digit 4: %.1f %%',accuracyDigit(5)),...
        sprintf('    digit 5: %.1f %%',accuracyDigit(6)),...
        sprintf('    digit 6: %.1f %%',accuracyDigit(7)),...
        sprintf('    digit 7: %.1f %%',accuracyDigit(8)),...
        sprintf('    digit 8: %.1f %%',accuracyDigit(9)),...
        sprintf('    digit 9: %.1f %%',accuracyDigit(10))};
    
end
