%-- This code has been realized to illustrate neural network concepts during 
%-- the workshop organized by the "club EEA" which happened in March 2018 
%-- https://rnn2018lyon.sciencesconf.org/
%--
%-- This code has been strongly inspired by the Mooc of Andrew Ng on coursera
%-- https://www.coursera.org/learn/machine-learning/
%-- 
%-- The application treated here concerns the automatic recognition of 
%-- digits drawn by hand. This code is not protected by any license, you 
%-- can re-use it freely without any restriction. Please do not forget to
%-- cite this event for any use of the code.

function keyPressDigit(~,~)

    %-- Global parameter for the GUI application
    global strTxt;
    global imgCanevas;
    global digitCanevas;
    
    
    %-- Press c to clear canevas
    if ( strcmp(get(gcf,'currentkey'),'c') )
        imgCanevas = zeros(180,180);    
        h1 = subplot(1,2,1); cla(h1);    
        subplot(1,2,1); imagesc(imgCanevas); axis image; colormap(gray);
        caxis([0 1]); axis off;        
        title({'Draw a digit here','Press c to clear || v to display info'}); 
        subplot(1,2,2); imagesc(zeros(40,40)); axis image; colormap(gray); caxis([0 1]); axis off;
    end
    
    %-- Press q to quit the application
    if ( strcmp(get(gcf,'currentkey'),'q') )
        h = gcf;
        close(h);
    end
    
    %-- Press v to display overall performance information
    if ( strcmp(get(gcf,'currentkey'),'v') )        
        digitDisplay = zeros(40,40);
        subplot(1,2,2); imagesc(digitDisplay); axis image; colormap(gray); 
        caxis([0 1]); axis off;
        text(20,20,strTxt,'Color','green','HorizontalAlignment','center');
    end    
    
    %-- Press s to save digit in a updated version of the database
    if ( strcmp(get(gcf,'currentkey'),'s') )      
        
        %-- use the current version of the canevas
        digit = digitCanevas;
        
        %-- ask for which corresponding digit to use as reference
        prompt = {'Enter digit value:'};
        titleF = 'Update Database';    
        dims = [1 35];
        definput = {'0'};
        answer = inputdlg(prompt,titleF,dims,definput);        
        value = str2num(answer{1});
        
        %-- load database
        local_data_path = '../data/';
        if (exist([local_data_path,'mnist_new.mat'],'file'))
            load([local_data_path,'mnist_new.mat']);
        else
            load([local_data_path,'mnist.mat']);
        end
        
        %-- store new digit
        training.images(:,:,end+1) = digit;
        training.labels(end+1,1) = value;
        training.count = size(training.images,3);
        save([local_data_path,'mnist_new.mat'],'training','testing')
        
    end
    
    %-- Press d to learn from the updated version of the database 
    if ( strcmp(get(gcf,'currentkey'),'d') )      
        
        %-- load database
        local_data_path = '../data/';
        if (exist([local_data_path,'mnist_new.mat'],'file'))
            load([local_data_path,'mnist_new.mat']);
        else
            return
        end        
        
        %-- Learn from updated database (cf. scriptTrainingNN for more details)
        local_param_path = '../param/';        
        widthDigit = size(training.images,2);
        heightDigit = size(training.images,1);
        m = size(training.images,3);
        X = zeros(m,widthDigit*heightDigit);
        for k=1:m
            digit = training.images(:,:,k);
            X(k,:) = digit(:)';
        end        
        y = training.labels;
        y(y==0) = 10;   %-- the digit 0 is identified as label 10
        disp('Training Neural Network...')
        %-- parameters
        maxIter = 100;              %-- maximum number of iterations
        input_layer_size  = 900;    %-- 30x30 Input Images of Digits
        hidden_layer_size = 25;     %-- 25 hidden units
        output_layer_size = 10;     %-- 10 units to recognize digits from 0 to 9        
        lambda = 1;                 % regularization parameter
        param = struct('input_layer_size',input_layer_size,...
            'hidden_layer_size',hidden_layer_size,...
            'output_layer_size',output_layer_size,'maxIter',maxIter,...
            'lambda',lambda);        
        [Theta1,Theta2] = nn.train(X, y, param);
        filename_param = 'param_mnist_new.mat';
        if (~exist(local_param_path,'dir'))
             mkdir(local_param_path);
        end
        save([local_param_path,filename_param],'Theta1','Theta2');
        disp(['New parameters saved to ',[local_param_path,filename_param]])        
        
        strTxt = visu.computeTxtInfo();
        digitDisplay = zeros(40,40);
        subplot(1,2,2); imagesc(digitDisplay); axis image; colormap(gray); 
        caxis([0 1]); axis off;
        text(20,20,strTxt,'Color','green','HorizontalAlignment','center');          
        
    end
    
end

