%-- This code has been realized to illustrate neural network concepts during 
%-- the workshop organized by the "club EEA" which happened in March 2018 
%-- https://rnn2018lyon.sciencesconf.org/
%--
%-- This code has been strongly inspired by the Mooc of Andrew Ng on coursera
%-- https://www.coursera.org/learn/machine-learning/
%-- 
%-- The application treated here concerns the automatic recognition of 
%-- digits drawn by hand. This code is not protected by any license, you 
%-- can re-use it freely without any restriction. Please do not forget to
%-- cite this event for any use of the code.

clear all;
close all;
clc;
addpath('../');


%-- parameters
maxIter = 100;                  %-- maximum number of iterations
input_layer_size  = 900;        %-- number of input units  (30x30 units to in our case)
hidden_layer_size = 25;         %-- number of hidden units
output_layer_size = 10;         %-- number of output units (10 units to recognize digits from 0 to 9 in our case)
lambda = 1;                     %-- regularization parameter
gradient_descent_scheme = 2;    %-- 1: simple gradient descente / 2: Polack-Ribiere conjugate gradient
param = struct('input_layer_size',input_layer_size,...
    'hidden_layer_size',hidden_layer_size,...
    'output_layer_size',output_layer_size,'maxIter',maxIter,...
    'lambda',lambda,'gradient_descent_scheme',gradient_descent_scheme);


%-- mnist-like database location
url = 'https://www.creatis.insa-lyon.fr/~bernard/workshop/';
local_data_path = '../data/';
local_param_path = '../param/';


%-- Downlad minst-like database
filename_db = 'mnist.mat';
if (~exist([local_data_path,filename_db],'file'))
     tools.download(filename_db,url,local_data_path);
end


%-- Load mnist-like database
load([local_data_path,filename_db]);
widthDigit = size(training.images,2);
heightDigit = size(training.images,1);


%-- total size of training database
m = size(training.images,3);


%-- Create X matrix
X = zeros(m,widthDigit*heightDigit);
for k=1:m
    digit = training.images(:,:,k);
    X(k,:) = digit(:)';
end


%-- Create y vector
y = training.labels;
y(y==0) = 10;   %-- the digit 0 is identified as label 10


%-- Randomly select 100 data points to display
rand_indices = randperm(m);
sel = X(rand_indices(1:100),:);
visu.displayDatabase(sel,widthDigit,heightDigit);


%-- Train neural network method
disp('Training Neural Network...')
[Theta1,Theta2] = nn.train(X, y, param);


%-- Save learned parameters
filename_param = 'param_mnist.mat';
if (~exist(local_param_path,'dir'))
     mkdir(local_param_path);
end
save([local_param_path,filename_param],'Theta1','Theta2');
disp(['Parameters saved to ',[local_param_path,filename_param]])


%-- Predict on the training dataset
pred = nn.predict(Theta1, Theta2, X);
disp(['Training Set Accuracy: ',num2str(mean(double(pred == y)) * 100)])

